#
# Script
#   selfsign.sh
#
# Description
#   Installs and configures vsftpd as a service, with the option to configure for FTPS (FTP over SSL).
#
#   Note that SFTP (FTP over SSH) is not supported.
#
# Usage
#   selfsign.sh commonname [certpath keypath orgunit org location state country]
#
#       commonname: Required. SSL Certificate Common Name.
#
#       certpath:   Optional. Path to output the certificate generated by openssl.
#                   Defaults to '/etc/ssl/certs/vsftpd.pem' if omitted.
#
#       keypath:    Optional. Path to output the key generated by openssl.
#                   Defaults to '/etc/ssl/private/vsftpd.key' if omitted.
#
#       orgunit:    Optional. SSL Certificate Organization Unit.
#                   Defaults to 'CDAF' if omitted.
#
#       org:        Optional. SSL Certificate Organization.
#                   Defaults to 'CDAF.io' if omitted.
#
#       location:   Optional. SSL Certificate Location.
#                   Defaults to 'Indian Ocean Territory' if omitted.
#
#       state:      Optional. SSL Certificate State.
#                   Defaults to 'Indian Ocean' if omitted.
#
#       country:    Optional. SSL Certificate Country.
#                   Defaults to 'IO' if omitted.
#

scriptName='selfsign.sh'
echo "[$scriptName] --- start ---"

commonname=$1
certpath=$2
keypath=$3
orgunit=$4
org=$5
location=$6
state=$7
country=$8

if [ -z "$certpath" ]; then certpath='/etc/ssl/certs/vsftpd.pem'; fi
if [ -z "$keypath" ];  then keypath='/etc/ssl/private/vsftpd.key'; fi
if [ -z "$orgunit" ];  then orgunit='CDAF'; fi
if [ -z "$org" ];      then org='CDAF.io'; fi
if [ -z "$location" ]; then location='Indian Ocean Territory'; fi
if [ -z "$state" ];    then state='Indian Ocean'; fi
if [ -z "$country" ];  then country='IO'; fi

echo "[$scriptName] Parameters"
echo "  commonname=$commonname"
echo "  certpath=$certpath"
echo "  keypath=$keypath"
echo "  orgunit=$orgunit"
echo "  org=$org"
echo "  location=$location"
echo "  state=$state"
echo "  country=$country"

if [ -z "$commonname" ]
then
    >&2 echo "[$scriptName] The commonname argument is required."
    exit 1
fi

# Install from global repositories only supporting CentOS and Ubuntu
echo "[$scriptName] Determine distribution, only Ubuntu/Debian and CentOS/RHEL supported"
uname -a
centos=$(uname -a | grep el)

echo "[$scriptName] Install base software (openssl)"
if [ -z "$centos" ]; then
	echo "[$scriptName] Ubuntu/Debian, update repositories using apt-get"
	sudo apt-get update
	sudo apt-get install -y openssl
else
	echo "[$scriptName] CentOS/RHEL, update repositories using yum"
	sudo yum check-update
	sudo yum install -y openssl
fi

echo "[$scriptName] Configure SSL Certificate"
sudo openssl req -x509 -nodes -days 365 -newkey rsa:4096 -keyout "$keypath" -out "$certpath" -subj "/C=$country/ST=$state/L=$location/O=$org/OU=$orgunit/CN=$commonname"

echo "[$scriptName] --- end ---"
exit 0
